@@ -5,7 +5,7 @@
 {-# LANGUAGE TupleSections       #-}
 {- |
    Module      : Text.Pandoc.Readers
-   Copyright   : Copyright (C) 2006-2023 John MacFarlane
+   Copyright   : Copyright (C) 2006-2024 John MacFarlane
    License     : GNU GPL, version 2 or above
 
    Maintainer  : John MacFarlane <jgm@berkeley.edu>
@@ -50,6 +50,8 @@ module Text.Pandoc.Readers
   , readTxt2Tags
   , readEPUB
   , readMuse
+  , readMan
+  , readMdoc
   , readFB2
   , readIpynb
   , readCSV
@@ -61,6 +63,8 @@ module Text.Pandoc.Readers
   , readRIS
   , readRTF
   , readTypst
+  , readDjot
+  , readPod
   -- * Miscellaneous
   , getReader
   , getDefaultExtensions
@@ -97,6 +101,7 @@ import Text.Pandoc.Readers.Native
 import Text.Pandoc.Readers.ODT
 import Text.Pandoc.Readers.OPML
 import Text.Pandoc.Readers.Org
+import Text.Pandoc.Readers.Pod
 import Text.Pandoc.Readers.RST
 import Text.Pandoc.Readers.Textile
 import Text.Pandoc.Readers.TikiWiki
@@ -104,6 +109,7 @@ import Text.Pandoc.Readers.TWiki
 import Text.Pandoc.Readers.Txt2Tags
 import Text.Pandoc.Readers.Vimwiki
 import Text.Pandoc.Readers.Man
+import Text.Pandoc.Readers.Mdoc
 import Text.Pandoc.Readers.CSV
 import Text.Pandoc.Readers.CslJson
 import Text.Pandoc.Readers.BibTeX
@@ -111,6 +117,7 @@ import Text.Pandoc.Readers.EndNote
 import Text.Pandoc.Readers.RIS
 import Text.Pandoc.Readers.RTF
 import Text.Pandoc.Readers.Typst
+import Text.Pandoc.Readers.Djot
 import qualified Text.Pandoc.UTF8 as UTF8
 import Text.Pandoc.Sources (ToSources(..), sourcesToText)
 
@@ -164,7 +171,10 @@ readers = [("native"       , TextReader readNative)
           ,("ris"          , TextReader readRIS)
           ,("rtf"          , TextReader readRTF)
           ,("typst"        , TextReader readTypst)
-           ]
+          ,("djot"         , TextReader readDjot)
+          ,("mdoc"         , TextReader readMdoc)
+          ,("pod"          , TextReader readPod)
+          ]
 
 -- | Retrieve reader, extensions based on format spec (format+extensions).
 getReader :: PandocMonad m => Format.FlavoredFormat -> m (Reader m, Extensions)
