@@ -2,7 +2,7 @@
 {-# LANGUAGE OverloadedStrings #-}
 {- |
    Module      : Text.Pandoc.Writers.LaTeX.Table
-   Copyright   : Copyright (C) 2006-2023 John MacFarlane
+   Copyright   : Copyright (C) 2006-2024 John MacFarlane
    License     : GNU GPL, version 2 or above
 
    Maintainer  : John MacFarlane <jgm@berkeley.edu>
@@ -33,11 +33,12 @@ import Text.Pandoc.Writers.LaTeX.Caption (getCaption)
 import Text.Pandoc.Writers.LaTeX.Notes (notesToLaTeX)
 import Text.Pandoc.Writers.LaTeX.Types
   ( LW, WriterState (stBeamer, stExternalNotes, stInMinipage, stMultiRow
-                    , stNotes, stTable) )
+                    , stNotes, stTable, stOptions) )
 import Text.Pandoc.Writers.LaTeX.Util (labelFor)
 import Text.Printf (printf)
 import qualified Text.Pandoc.Builder as B
 import qualified Text.Pandoc.Writers.AnnotatedTable as Ann
+import Text.Pandoc.Options (CaptionPosition(..), WriterOptions(..))
 
 tableToLaTeX :: PandocMonad m
              => ([Inline] -> LW m (Doc Text))
@@ -45,8 +46,13 @@ tableToLaTeX :: PandocMonad m
              -> Ann.Table
              -> LW m (Doc Text)
 tableToLaTeX inlnsToLaTeX blksToLaTeX tbl = do
+  opts <- gets stOptions
   let (Ann.Table (ident, _, _) caption specs thead tbodies tfoot) = tbl
   CaptionDocs capt captNotes <- captionToLaTeX inlnsToLaTeX caption ident
+  let hasTopCaption = not (isEmpty capt) &&
+                        writerTableCaptionPosition opts == CaptionAbove
+  let hasBottomCaption = not (isEmpty capt) &&
+                          writerTableCaptionPosition opts == CaptionBelow
   let isSimpleTable =
         all ((== ColWidthDefault) . snd) specs &&
         all (all isSimpleCell)
@@ -64,24 +70,31 @@ tableToLaTeX inlnsToLaTeX blksToLaTeX tbl = do
   -- duplicate the header rows for this.
   head' <- do
     let mkHead = headToLaTeX blksToLaTeX isSimpleTable colCount
-    case (not $ isEmpty capt, not $ isEmptyHead thead) of
-      (False, False) -> return "\\toprule\\noalign{}"
-      (False, True)  -> mkHead thead
-      (True, False)  -> return (capt $$ "\\toprule\\noalign{}" $$ "\\endfirsthead")
-      (True, True)   -> do
+    case (hasTopCaption, isEmptyHead thead) of
+      (False, True) -> return "\\toprule\\noalign{}"
+      (False, False)  -> mkHead thead
+      (True, True)  -> return (capt <> "\\tabularnewline"
+                                $$ "\\toprule\\noalign{}"
+                                $$ "\\endfirsthead")
+      (True, False)   -> do
         -- avoid duplicate notes in head and firsthead:
         firsthead <- mkHead thead
         repeated  <- mkHead (walk removeNote thead)
-        return $ capt $$ firsthead $$ "\\endfirsthead" $$ repeated
+        return $ capt <> "\\tabularnewline"
+                 $$ firsthead
+                 $$ "\\endfirsthead"
+                 $$ repeated
   rows' <- mapM (rowToLaTeX blksToLaTeX isSimpleTable colCount BodyCell) $
                 mconcat (map bodyRows tbodies)
-  foot' <- if isEmptyFoot tfoot
-           then pure empty
-           else do
-             lastfoot <- mapM
-                (rowToLaTeX blksToLaTeX isSimpleTable colCount BodyCell) $
-                footRows tfoot
-             pure $ "\\midrule\\noalign{}" $$ vcat lastfoot
+  lastfoot <- mapM (rowToLaTeX blksToLaTeX isSimpleTable colCount BodyCell) $
+                    footRows tfoot
+  let foot' = (if isEmptyFoot tfoot
+                  then mempty
+                  else "\\midrule\\noalign{}" $$ vcat lastfoot)
+              $$ "\\bottomrule\\noalign{}"
+              $$ (if hasBottomCaption
+                     then "\\tabularnewline" $$ capt
+                     else mempty)
   modify $ \s -> s{ stTable = True }
   notes <- notesToLaTeX <$> gets stNotes
   beamer <- gets stBeamer
@@ -99,10 +112,8 @@ tableToLaTeX inlnsToLaTeX blksToLaTeX tbl = do
        (if beamer
              then [ vcat rows'
                   , foot'
-                  , "\\bottomrule\\noalign{}"
                   ]
              else [ foot'
-                  , "\\bottomrule\\noalign{}"
                   , "\\endlastfoot"
                   ,  vcat rows'
                   ])
@@ -137,17 +148,19 @@ colDescriptors isSimpleTable
                        then replicate (length specs)
                             (1 / fromIntegral (length specs))
                        else map toRelWidth widths
-  in if defaultWidthsOnly && isSimpleTable
-     then hcat $ map (literal . colAlign) aligns
-     else (cr <>) . nest 2 . vcat . map literal $
-          zipWith (toColDescriptor (length specs))
-                  aligns
-                  relativeWidths
+  in if null aligns
+        then "l"  -- #9350, table needs at least one column spec
+        else if defaultWidthsOnly && isSimpleTable
+                then hcat $ map (literal . colAlign) aligns
+                else (cr <>) . nest 2 . vcat . map literal $
+                     zipWith (toColDescriptor (length specs))
+                             aligns
+                             relativeWidths
   where
     toColDescriptor :: Int -> Alignment -> Double -> Text
     toColDescriptor numcols align width =
       T.pack $ printf
-      ">{%s\\arraybackslash}p{(\\columnwidth - %d\\tabcolsep) * \\real{%0.4f}}"
+      ">{%s\\arraybackslash}p{(\\linewidth - %d\\tabcolsep) * \\real{%0.4f}}"
       (T.unpack (alignCommand align))
       ((numcols - 1) * 2)
       width
@@ -190,7 +203,6 @@ captionToLaTeX inlnsToLaTeX caption ident = do
                        else "\\caption" <> captForLot <>
                             braces captionText
                             <> label
-                            <> "\\tabularnewline"
     }
 
 type BlocksWriter m = [Block] -> LW m (Doc Text)
@@ -354,12 +366,20 @@ cellToLaTeX blockListToLaTeX isSimpleTable colCount celltype annotatedCell = do
                                <> braces (literal colDescr)
                                <> braces ("%\n" <> x)
                                   -- linebreak for readability
+  let hasColWidths = not (all (== ColWidthDefault) colWidths)
+  let aligncmd = case specAlign of
+                   AlignCenter -> "\\centering\\arraybackslash "
+                   AlignRight -> "\\raggedright\\arraybackslash "
+                   _ -> mempty
   let inMultiRow x = case rowspan of
                        (RowSpan 1) -> x
                        (RowSpan n) -> let nrows = literal (tshow n)
                                       in "\\multirow" <> braces nrows
-                                         <> braces "=" -- width of column
-                                         <> braces x
+                                         <> braces -- width of column
+                                             (if hasColWidths
+                                                 then "=" -- max width
+                                                 else "*") -- natural width
+                                         <> braces (aligncmd <> x)
   return . inMultiColumn . inMultiRow $ result
 
 -- | Returns the width of a cell spanning @n@ columns.
@@ -389,7 +409,7 @@ multicolumnDescriptor isSimpleTable
               (T.unpack (colAlign align))
               (if colnum + colspan >= numcols then skipColSep else "")
 
-        else printf "%s>{%s\\arraybackslash}p{(\\columnwidth - %d\\tabcolsep) * \\real{%0.4f} + %d\\tabcolsep}%s"
+        else printf "%s>{%s\\arraybackslash}p{(\\linewidth - %d\\tabcolsep) * \\real{%0.4f} + %d\\tabcolsep}%s"
               (if colnum == 0 then skipColSep else "")
               (T.unpack (alignCommand align))
               (2 * (numcols - 1))
